%GRADIENT_BINNING - Compute gradient features and bin them according to orientation angle.

%   Author: Uwe Schmidt, TU Darmstadt (uwe.schmidt@gris.tu-darmstadt.de)
%
%   This file is part of the implementation as described in the CVPR 2012 paper:
%   Uwe Schmidt and Stefan Roth. Learning Rotation-Aware Features: From Invariant Priors to Equivariant Descriptors.
%   Please see the file LICENSE.txt for the license governing this code.

function binned = gradient_binning(I,nbins,unsigned)
  if ~exist('unsigned','var'), unsigned = 1; end

  % image gradients, centered on the inside, uncentered at boundary pixels
  grad_x = [2*(I(:,2)-I(:,1)), conv2(I,[-1,0,1], 'valid'), 2*(I(:,end)-I(:,end-1))];
  grad_y = [2*(I(2,:)-I(1,:)); conv2(I,[-1,0,1]','valid'); 2*(I(end,:)-I(end-1,:))];

  angles = atan2(grad_y, grad_x);
  magnit = sqrt(grad_y.^2 + grad_x.^2);
  
  if unsigned
    st = pi/nbins;                                         % angle steps (unsigned)
    thr = 0:st:pi;                                         % angle thresholds (unsigned)
    angles = angles + pi;                                  % unsigned angles
    aidx = angles >= pi; angles(aidx) = angles(aidx) - pi; % make sure angles are between 0 and pi
  else
    st = (2*pi)/nbins; % angle steps (signed)
    thr = -pi:st:pi;   % angle thresholds (signed)
  end
  
  binned = binning_fast(angles,thr,1,1);
  binned = bsxfun(@times, binned, magnit(:));
  binned = reshape(binned,[size(I),nbins]);
  
end