%IHOF - Invariant histogram of oriented features.
%
%   See also HOF.

%   Author: Uwe Schmidt, TU Darmstadt (uwe.schmidt@gris.tu-darmstadt.de)
%
%   This file is part of the implementation as described in the CVPR 2012 paper:
%   Uwe Schmidt and Stefan Roth. Learning Rotation-Aware Features: From Invariant Priors to Equivariant Descriptors.
%   Please see the file LICENSE.txt for the license governing this code.

function H = ihof(binned,nrings,ncells_per_ring,do_normalization)

  nbins = size(binned,3);
  function H = post(cells,cells_norm)
    F = fft2(cells_norm);
    H = abs(F); % magnitude
    % include central cell
    h = abs(fft(reshape(cells(1,1,:),nbins,1)));
    h = h(1:ceil((nbins+1)/2)); % remove redundant fft coefficients
    H = [h(:); H(:)];
    H = H / max(H(:)); % scale data from 0 to 1
  end
    
  H = hof(binned,nrings,ncells_per_ring,do_normalization,@post);

end
