%RCRBM_ACTIVATION_BINNING - Compute RC-RBM features for several orientation angles.

%   Author: Uwe Schmidt, TU Darmstadt (uwe.schmidt@gris.tu-darmstadt.de)
%
%   This file is part of the implementation as described in the CVPR 2012 paper:
%   Uwe Schmidt and Stefan Roth. Learning Rotation-Aware Features: From Invariant Priors to Equivariant Descriptors.
%   Please see the file LICENSE.txt for the license governing this code.

function binned = rcrbm_activation_binning(cliques,imdims,rcrbm,unsigned)
  
  BW     = rcrbm.B * rcrbm.W;
  nbins  = rcrbm.ntforms;
  frdims = imdims - (rcrbm.fdims - 1);
  ph     = zeros(prod(frdims), rcrbm.nfilters, rcrbm.ntforms);

  for a = 1:rcrbm.ntforms
    ph(:,:,a) = cliques' * (rcrbm.T{a}*BW);
  end
  ph = logistic(ph);
    
  binned = zeros([frdims,nbins,rcrbm.nfilters]);
  for i = 1:rcrbm.nfilters
    binned(:,:,:,i) = reshape(ph(:,i,:),[frdims,nbins]);    
    % non-maximum suppression (hard-binning)
    [maxval,maxidx] = max(reshape(binned(:,:,:,i),prod(frdims),nbins),[],2);
    tmp = binning_fast(maxidx,1-1/2:nbins+1/2,0,0);
    tmp = bsxfun(@times, tmp, maxval(:));
    binned(:,:,:,i) = reshape(tmp,[frdims,nbins]);
  end
  
  if unsigned
    assert(mod(nbins,2)==0)
    binned = max(binned(:,:,(1:nbins/2),:), binned(:,:,(1:nbins/2)+nbins/2,:));
  end
  
end

function v = logistic(x)
  v = 1 ./ (1 + exp(-x));
end